@@ -1,11 +1,8 @@
 /*
  *   cddbd - CD Database Protocol Server
  *
- *   Copyright (C) 1996-1997  Steve Scherf
- *   Email: steve@moonsoft.com
- *   Moondog Software Productions - makers of fine public domain software.
- *
- *   21.09.1999 modified to comply protocol version 4 -OGF-
+ *   Copyright (C) 1996-1997  Steve Scherf (steve@moonsoft.com)
+ *   Portions Copyright (C) 1999-2006  by various authors
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,55 +16,39 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
 
+#ifndef __CDDBD_H__
+#define __CDDBD_H__
+
 #ifndef LINT
-static char *_cddbd_h_ident_ = "@(#)$Id: cddbd.h,v 1.18 2001/09/13 18:00:01 joerg78 Exp $";
+static char *const _cddbd_h_ident_ = "@(#)$Id: cddbd.h,v 1.43.2.8 2006/04/19 16:49:05 joerg78 Exp $";
 #endif
 
-/* OS dependencies */
-#ifdef __FreeBSD__
-#define HAS_UINT32_T
-#endif /* __FreeBSD__ */
+/* includes */
+#include <sys/types.h>
+#include <stdio.h>
+#include "list.h"
+#include "configurables.h"
 
-#ifdef __NetBSD__
+/* OS dependencies */
+#if defined __FreeBSD__ || defined __NetBSD__ || defined __OpenBSD__ || (defined __SVR4 && defined __sun)
 #define HAS_UINT32_T
-#endif /* __NetBSD__ */
+#endif /* defined __FreeBSD__ || defined __NetBSD__ || defined __OpenBSD__ || (defined __SVR4 && defined __sun) */
 
 #ifdef __linux__
 #include <stdint.h>
 #define HAS_UINT32_T
 #endif /* __linux__ */
 
-#ifdef __hpux
+#if defined __CYGWIN32__ || defined __host_mips || defined __hpux
 #define HAS_UINT32_T
-#endif /* __hpux */
-
-#ifdef __alpha__
-/* Oh my god, this is ugly!
- * The cddbd code uses type casts all over the place which is not
- * particularily good style.  Unfortunately, this doesn't work too
- * well on Alphas, so these functions are provided as an impedance
- * match.
- */
-/* Alpha: (int) is 32 bit; (long) and (void *) are 64 bit */
-#define ptr_to_uint32(x)         ((int) (((long) (x)) & 0x00000000FFFFFFFF))
-#define to_uint32(x)             ((int) ((x) & 0x00000000FFFFFFFF))
-#define errno_as_ptr             ((void *) (long) errno)
-#define ptr_as_uint(x)           ((unsigned long) (x))
-#define uint_as_ptr(x)           ((void *) (unsigned long) (x))
-#define ino_t_as_ptr(x)          ((void *) (long) (x))
-#else /* __alpha__ */
-/* most other CPUs: (int), (long) and (void *) are all 32 bit */
-#define ptr_to_uint32(x)         ((unsigned int) (x))
-#define to_uint32(x)             ((int) x)
-#define errno_as_ptr             ((void *) errno)
-#define ptr_as_uint(x)           ((unsigned int) (x))
-#define uint_as_ptr(x)           ((void *) (unsigned int) (x))
-#define ino_t_as_ptr(x)          ((void *) (int) (x))
-#endif /* __alpha__ */
+#ifndef socklen_t
+#define socklen_t int
+#endif
+#endif /*defined __CYGWIN32__ || defined __host_mips || defined __hpux */
 
 #ifdef _AIX
 #ifndef socklen_t
@@ -77,18 +58,36 @@ static char *_cddbd_h_ident_ = "@(#)$Id: cddbd.h,v 1.18 2001/09/13 18:00:01 joer
 void bzero (void *b, size_t len);
 #endif /* _AIX */
 
-#ifdef __osf__
+#if defined __osf__ || defined __APPLE__
 #ifndef socklen_t
-/* OSF1 V4.0 does not know socklen_t */
 #define socklen_t int
 #endif
 #endif /* __osf__ */
 
-#ifdef __CYGWIN32__
-#ifndef socklen_t
-#define socklen_t int
-#endif
-#endif /* __CYGWIN32__ */
+#if defined __alpha__ || defined __x86_64__
+/* Oh my god, this is ugly!
+ * The cddbd code uses type casts all over the place which is not
+ * particularily good style.  Unfortunately, this doesn't work too
+ * well on Alpha and AMD64, so these functions are provided as an
+ * impedance match.
+ */
+/* Alpha and AMD64: (int) is 32 bit; (long) and (void *) are 64 bit */
+#define ptr_to_uint32(x)         ((int) (((long) (x)) & 0x00000000FFFFFFFF))
+#define to_uint32(x)             ((int) ((x) & 0x00000000FFFFFFFF))
+#define errno_as_ptr             ((void *) (long) errno)
+#define ptr_as_uint(x)           ((unsigned long) (x))
+#define uint_as_ptr(x)           ((void *) (unsigned long) (x))
+#define ino_t_as_ptr(x)          ((void *) (long) (x))
+#else /* __alpha__ || __x86_64__ */
+/* most other CPUs: (int), (long) and (void *) are all 32 bit */
+#define ptr_to_uint32(x)         ((unsigned int) (x))
+#define to_uint32(x)             ((int) x)
+#define errno_as_ptr             ((void *) errno)
+#define ptr_as_uint(x)           ((unsigned int) (x))
+#define uint_as_ptr(x)           ((void *) (unsigned int) (x))
+#define ino_t_as_ptr(x)          ((void *) (int) (x))
+#endif /* __alpha__ || __x86_64__ */
+
 
 /* Preprocessor definitions. */
 
@@ -105,14 +104,18 @@ void bzero (void *b, size_t len);
 #define SPUT(c, buf) (buf)[cc] = c, cc++
 
 /* Fuzzy table header values. */
-#define FUZZY_MAGIC	0x32FD91C4
-#define FUZZY_VERSION	2
+#define FUZZY_MAGIC	    0x32FD91C5
+#define FUZZY_VERSION	3 
+
+/* Define empty (default) discid */
+#define DISCID_EMPTY	0x00000000
 
 /* Protocol flags. */
 #define P_QUOTE		0x00000001
 #define P_SITES		0x00000002
 #define P_QUERY		0x00000004
 #define P_READ		0x00000008
+#define P_UTF8		0x00000010
 
 #define PROTO_ENAB(flag)	(proto_flags[level] & (flag))
 
@@ -160,6 +163,17 @@ void bzero (void *b, size_t len);
 #define LOG_ALL		0xFFFFFFFF
 #define LOG_SUPER	0x0000001F	/* Superflag mask. */
 
+/* Values of file_charset. */
+#define FC_ONLY_ISO	1
+#define FC_PREFER_ISO	2
+#define FC_PREFER_UTF	3
+#define FC_ONLY_UTF	4
+
+/* Values of utf_as_iso. */
+#define UAI_ACCEPT	1
+#define UAI_REJECT	2
+#define UAI_CONVERT	3
+
 /* Remote ops. */
 #define RMT_OP_NONE	-1
 #define RMT_OP_CATCHUP	0
@@ -198,6 +212,9 @@ void bzero (void *b, size_t len);
 #define DF_DONE		0x00000004	/* DB entry has been read. */
 #define DF_SUBMITTER	0x00000008	/* Entry has a "submitted via" field. */
 #define DF_CK_SUBMIT	0x00000010	/* Check submitter. */
+#define DF_ENC_LATIN1	0x00000020	/* Allow input in ISO-8859-1. */
+#define DF_ENC_UTF8	0x00000040	/* Allow input in UTF-8. */
+#define DF_WIN		0x00000080	/* File is a "Windows format" database file. */
 
 /* DB parsing phases. */
 #define DP_COMMENT	0
@@ -212,7 +229,7 @@ void bzero (void *b, size_t len);
 #define DP_NPHASE	9
 
 /* Mail flags. */
-#define MF_NONE         0x00000000      /* nothing. */
+#define MF_NONE		0x00000000	/* nothing. */
 #define MF_FAIL		0x00000001	/* Failed submission email. */
 #define MF_TEST		0x00000002	/* Test submission email. */
 #define MF_MULTI	0x00000004	/* Mail should be multipart. */
@@ -238,8 +255,10 @@ void bzero (void *b, size_t len);
 #define EC_COMMAND	2
 
 /* Charset types (only a subset). */
+/* zeke - changes on these defines... */
 #define CC_US_ASCII	0
-#define CC_ISO_8859_1	2
+#define CC_ISO_8859	1
+#define CC_UTF_8	2
 
 /* Submitter revision types, in ranked order. */
 #define ST_DEVELOP	0
@@ -282,6 +301,12 @@ void bzero (void *b, size_t len);
 #define LK_UPDATE	6
 #define LK_NUMLOCKS	7
 
+#ifdef DB_WINDOWS_FORMAT
+/* Category-specific lock names. */
+#define CATLK_MERGE	0 /* Merge lock */
+#define CATLK_NUMLOCKS	1
+#endif
+
 /* Encoding errors. */
 #define EN_OK		0
 #define EN_ERROR	1
@@ -305,7 +330,7 @@ void bzero (void *b, size_t len);
 /* Constants. */
 #define CDDBHOSTNAMELEN	64	/* Max host name length. */
 #define DBLINESIZ   	256	/* Max number of chars per line in a DB entry. */
-#define CDDBBUFSIZ	256	/* Max input buffer size. */
+#define CDDBBUFSIZ	2048	/* Max input buffer size. */
 #define CDDBARGSIZ	128	/* Max input arg size. */
 #define CDDBMAXARGS	128	/* Max # of args for a CDDBP command. */
 #define CDDBCMDLEN	2048	/* Max size of a CDDBP command. */
@@ -317,7 +342,11 @@ void bzero (void *b, size_t len);
 #define CDDBPLBLSIZ	16	/* Max password label size. */
 #define CDDBMAXTMP	10	/* Max # of attempts at getting a temp file. */
 #define CDDBMAXTRK	99	/* Max number of tracks on a CD. */
+#ifdef DB_WINDOWS_FORMAT
+#define CDDBLCKNAMLEN	41	/* Length of lock file prefix. */
+#else
 #define CDDBLCKNAMLEN	8	/* Length of lock file prefix. */
+#endif
 #define CDDBUINTLEN	8	/* Length of a 32 bit ascii unsigned int. */
 #define CDDBXCRCLEN	16	/* Length of a Mime ascii CRC. */
 #define CDDBXECHOLEN	64	/* Length of a Mime echo string. */
@@ -328,10 +357,12 @@ void bzero (void *b, size_t len);
 #define CDDBMAXDBDIR	50	/* Max number of database subdirectories. */
 #define CDDBMAXERR	3	/* Max number of allowable errors. */
 #define CDDBMINREV	0	/* Min entry revision level. */
+#define MAX_QUITS	3	/* Max number of reentrant quit calls. */ 
 #define MAX_LOG_LIST	30	/* Max number of lines in a log list. */
 #define MIN_PROTO_LEVEL	1	/* Minimum protocol level supported. */
-#define MAX_PROTO_LEVEL	5	/* Protocol level this server supports. */
+#define MAX_PROTO_LEVEL	6	/* Protocol level this server supports. */
 #define DEF_PROTO_LEVEL	1	/* Default protocol level. */
+#define UNICODE_LEVEL	6	/* Lowest level that uses UTF-8. */
 #define DEF_XMIT_TO	60	/* Default transmit timeout. */
 #define DEF_INPUT_TO	60	/* Default input timeout. */
 #define DEF_ACCESS_TO	60	/* Default access timeout. */
@@ -354,6 +385,7 @@ void bzero (void *b, size_t len);
 #define DEF_LOCK_TIME	250	/* Default lock retry time (ms). */
 #define DEF_LOCK_WAIT	60	/* Default lock wait time (sec). */
 #define DEF_STRIP_EXT	0	/* Default extended data strip value. */
+#define DEF_CK_BERZERK	1	/* Default "check berzerk" value. */ 
 
 #define MIN_CHECK_LEVEL	1	/* Min DB checking level. */
 #define MAX_CHECK_LEVEL	1	/* Max DB checking level. */
@@ -362,6 +394,14 @@ void bzero (void *b, size_t len);
 #define FL_REPAIR	2	/* DB checking level for repairing entries. */
 #define FL_REMOVE	3	/* DB checking level for removing entries. */
 
+#ifdef DB_WINDOWS_FORMAT
+#define WINFILEIDLEN	6		/* filename length in windows db e.g. 01to23 */ 
+#define STARTTAG	"#FILENAME="	/* discid tag in "windows" file */
+#define STARTTAGLEN	10		/* length of the tag */
+#define WINMAXPOSTRECORDS 1000		/* maximum number of records when posting before a merge */
+#endif /* DB_WINDOWS_FORMAT */
+
+
 
 /* Structure definitions. */
 
@@ -404,8 +444,8 @@ typedef struct log {
 	unsigned int	flag;		/* Flag. */
 	int		cnt;		/* Message count. */
 	int		dflag;		/* Display flags. */
-	char	        *name;		/* Log flag name. */
-	char	        *banr;		/* Banner. */
+	char		*name;		/* Log flag name. */
+	char		*banr;		/* Banner. */
 } log_t;
 
 
@@ -473,6 +513,19 @@ typedef struct month {
 } month_t;
 
 
+/* zeke - mini email hdr for parsing in db struct */
+typedef struct email_hdr_db {
+	int eh_valid;
+	int eh_flags;
+	int eh_charset;
+	int eh_encoding;
+	char eh_to[CDDBBUFSIZ];
+	char eh_rcpt[CDDBBUFSIZ];
+	char eh_subj[CDDBBUFSIZ];
+	char eh_host[CDDBBUFSIZ];
+} email_hdr_db_t;
+
+
 /* Defines for db_t flags. */
 #define DB_DISCLEN	0x00000001
 #define DB_OFFSET	0x00000002
@@ -481,12 +534,16 @@ typedef struct month {
 #define DB_PROCESSOR	0x00000010
 #define DB_STRIP	0x00000020
 #define DB_EMPTY	0x00000040
+#define DB_ENC_ASCII	0x00000080	/* Data valid as US-ASCII. */
+#define DB_ENC_LATIN1	0x00000100	/* Data valid as ISO-8859-1. */
+#define DB_ENC_UTF8	0x00000200	/* Data valid as UTF-8. */
 
 /* Defines for db_t.db_rev. */
 #define DB_MAX_REV	-1
 
 /* Database entry struct. */
 typedef struct db {
+	email_hdr_db_t db_eh;		/* zeke - added for eh transfer... */
 	int db_flags;
 	int db_rev;
 	int db_offset[CDDBMAXTRK];
@@ -547,9 +604,12 @@ typedef struct coord {
 #define ST_PROTO_SMTP	2
 #define ST_PROTO_GROUP	3
 
-#define ST_FLAG_NOXMIT	0x00000001
-#define ST_FLAG_PWDLBL	0x00000002
-#define ST_FLAG_OMIME	0x00000004
+#define ST_FLAG_NOXMIT	0x00000001 /* do not transmit to this site */
+#define ST_FLAG_PWDLBL	0x00000002 /* need to use passwd for authorizing transmissions */
+#define ST_FLAG_OMIME	0x00000004 /* do not use multipart e-mails (QP & BASE64) for transmission */
+#define ST_FLAG_OFIELDS	0x00000008 /* send only original xmcd file format fields (no DGENRE/DYEAR) */
+#define ST_FLAG_ONLYISO	0x00000010 /* send transmission e-mails in utf-8 */
+
 
 typedef struct csite {
 	char st_name[CDDBHOSTNAMELEN + 1];
@@ -574,11 +634,9 @@ typedef struct encode {
 
 
 /* Charset info. */
-#define CT_8BIT		0x00000001
-
 typedef struct charset {
-	char *ct_name;
-	int ct_flags;
+	char *name;
+	int df_flags;
 } charset_t;
 
 
@@ -596,12 +654,13 @@ typedef struct fuzzy_hdr {
 /* Fuzzy table entry data. */
 typedef struct fuzzy_data {
 	unsigned int fd_discid;	/* Disc ID. */
-	int fd_trks;		/* Track count. */
-	short fd_disclen;	/* Length of disc in seconds. */
-	short fd_catind;	/* Category index. */
+	unsigned int fd_trks : 7;		/* Track count. */
+	unsigned int fd_disclen : 13;	/* Length of disc in seconds. */
+	unsigned int fd_catind : 4;	/* Category index. */
 } fdata_t;
 
 
+#define OFFSET_BIT_WIDTH 19
 /* Fuzzy table entry. */
 typedef struct fuzzy_hash {
 	fdata_t fh_s;
@@ -776,6 +835,15 @@ typedef struct ct_data {
 	int cd_data[CT_DATA_LEN];
 } ct_data_t;
 
+/* CRC function definitions. */
+#define CRC_START	0
+#define CRC_END		1
+#define CRC_COMPUTE	2
+#define CRC_ONCE	3
+#define CRC_STRING	4
+#define CRC_FILE	5
+#define CRC_DESC	6
+
 /* Assumes 32-bit integers. */
 #ifndef HAS_UINT32_T
 typedef unsigned int uint32_t;
@@ -828,12 +896,14 @@ void sighand(int);
 void smtp_close(void);
 void strip_crlf(char *);
 void fmatch_free(void *p);
+void convert_db_charset(db_t *);
+void convert_db_charset_proto(db_t *, int);
 
 int asy_encode(char *, char **);
 int asy_mappable(unsigned char);
 int *cddbd_count(void);
 int categ_index(char *);
-int cddbd_async_command(char *);
+int cddbd_async_command(char *, int);
 int cddbd_charcasecmp(char *, char *);
 int cddbd_count_history(char *);
 int cddbd_close_history(void);
@@ -852,13 +922,18 @@ int cddbd_write_ulist(char *, char *);
 int cddbp_log_stats(void);
 int cddbp_command(char *, int);
 int cddbp_open(csite_t *);
-int cddbp_transmit(db_t *, char *, unsigned int);
+int cddbp_setproto(void);
+int cddbp_transmit(db_t *, char *, unsigned int, int);
 int cddbp_update(void);
 int ck_client_perms(char *, char *, int);
 int crc_email(uint32_t *, FILE *, char *, char *, char *);
 int db_post(db_t *, char *, unsigned int, char *);
 int db_strip(db_t *);
+#ifdef DB_WINDOWS_FORMAT
+int db_write(FILE *fp, db_t *, int level, unsigned int discid);
+#else
 int db_write(FILE *fp, db_t *, int level);
+#endif
 int get_serv_index(char *);
 int fp_copy_buf(FILE *, char *);
 int is_blank(char *, int);
@@ -867,12 +942,15 @@ int is_dot(char *);
 int is_dbl_dot(char *);
 int is_fuzzy_match(int *, int *, int, int, int);
 int is_instr(char, char *);
+int is_DblHash(char *);
 int is_parent_dir(char *);
 int is_qp_hex(unsigned char);
 int is_rfc_1521_mappable(unsigned char *, int, int);
 int is_rfc_1521_print(unsigned char);
 int is_space(char);
-int is_valid(char *);
+int is_valid_ascii(char *);
+int is_valid_latin1(char *);
+int is_valid_utf8(char *);
 int is_wspace(int);
 int match_host(char *);
 int octet_to_char(unsigned char *, char);
@@ -884,12 +962,19 @@ int rfc_1521_qp_decode(int, unsigned char *, unsigned char **, int *);
 int setsitegrp(char *);
 int smtp_command(csite_t *, char *, int, char *, char *);
 int smtp_open(void);
-int smtp_transmit(FILE *, char *, char *, char *, char *, char *, int, int,
-    ct_key_t *);
+int smtp_transmit(FILE *, int, char *, char *, char *, char *, char *, int, int,
+    		  ct_key_t *);
 int validate_email(FILE *, email_hdr_t *, char *);
+int return_mail(FILE *, email_hdr_t *, int, char *);
+int parse_utf8(const char **ps);
+
 
 struct tm *date_to_tm(char *);
 
+char *cddbd_dirname(char *);
+#ifdef DB_WINDOWS_FORMAT
+char *cddbd_basename(char *path);
+#endif
 char *cddbd_gets(char *, int);
 char *cddbd_mktemp(void);
 char *cddbd_strcasestr(char *, char *);
@@ -914,27 +999,36 @@ csite_t *getsitenam(char *, int);
 
 lhead_t *fuzzy_search(int, int *, int);
 
+FILE * db_prepare_unix_read(char *);
+int db_skip(FILE *, char *);
+
 db_t *db_read(FILE *, char *, int);
 
 clck_t *cddbd_lock_alloc(char *);
 
 hperm_t *ck_host_perms(int);
 
+/* crc.c */
+int crc32(int, void *, uint32_t *, uint32_t *);
+
 
 
 /* External definitions. */
-
+extern unsigned int db_curr_discid;
 extern int categcnt;
 extern int cgi;
 extern int db_dir_mode;
 extern int db_errno;
 extern int db_file_mode;
 extern int delay_time;
+extern int debug; /* need to export it as well */
 extern int dir_mode;
 extern int dup_ok;
 extern int dup_policy;
 extern int elapse_time;
 extern int email_time;
+extern int file_charset;
+extern int file_df_flags;
 extern int file_mode;
 extern int fuzzy_div;
 extern int fuzzy_factor;
@@ -946,6 +1040,7 @@ extern int prclen;
 extern int strip_ext;
 extern int sublen;
 extern int trklen;
+extern int utf_as_iso;
 extern int verbose;
 extern int xmit_time;
 
@@ -1021,10 +1116,30 @@ extern uid_t db_uid;
 extern uid_t uid;
 
 extern clck_t *locks[LK_NUMLOCKS];
+#ifdef DB_WINDOWS_FORMAT
+extern clck_t ***categlocks;
+#endif
 
 extern proto_t proto_flags[];
 extern ctimer_t timers[];
 extern encode_t encoding_types[];
-extern charset_t char_types[];
+extern charset_t charsets[];
 
 extern hperm_t hperm;
+
+/* String charset conversion functions defined in charset.c. */
+void charset_latin1_utf8(const char *s, char **p);
+int charset_utf8_latin1(const char *s, char **p);
+int charset_is_utf8(const char *s);
+int charset_is_valid_utf8(const char *s);
+int charset_is_valid_ascii(const char *s);
+int charset_is_valid_latin1(const char *s);
+
+/* DB charset conversion functions defined in db.c. */
+int db_utf8_latin1(db_t *db);
+int db_utf8_latin1_exact(db_t *db);
+void db_latin1_utf8(db_t *db);
+int db_looks_like_utf8(db_t *db);
+int db_disam_charset(db_t *db);
+
+#endif
