@@ -1,12 +1,11 @@
 #!/bin/sh
 #
-#   @(#)$Id: config.sh,v 1.3 1996/12/22 01:48:54 steve Exp $
+#   @(#)$Id: config.sh,v 1.18.2.3 2006/04/14 14:45:06 joerg78 Exp $
 #
 #   cddbd - CD Database Protocol Server
 #
-#   Copyright (C) 1996  Steve Scherf
-#   Email: steve@moonsoft.com
-#   Moondog Software Productions - makers of fine public domain software.
+#   Copyright (C) 1996       Steve Scherf (steve@moonsoft.com)
+#   Portions Copyright (C) 2001-2006  by various authors 
 #
 #   Based on the original source by Ti Kan.
 #
@@ -22,24 +21,63 @@
 #
 #   You should have received a copy of the GNU General Public License
 #   along with this program; if not, write to the Free Software
-#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
 PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc:/usr/local/bin
 export PATH
 
-CDDBD_VER=1.3.1
-DIRPERM=755
-FILEPERM=644
-BINPERM=711
+CDDBD_VER=1.5.2
 UMASK=022
 ERRFILE=/tmp/cddbd.err
 TMPFILE=/tmp/cddbd.$$
+CONFIGHEADER=configurables.h
 
 #
 # Utility functions
 #
 
+getyn()
+{
+	if [ -z "$YNDEF" ]
+	then
+		YNDEF=y
+	fi
+
+	while :
+	do
+		$ECHO "$*? [${YNDEF}] \c"
+		read ANS
+		if [ -n "$ANS" ]
+		then
+			case $ANS in
+			[yY])
+				RET=0
+				break
+				;;
+			[nN])
+				RET=1
+				break
+				;;
+			*)
+				$ECHO "Please answer y or n"
+				;;
+			esac
+		else
+			if [ $YNDEF = y ]
+			then
+				RET=0
+			else
+				RET=1
+			fi
+			break
+		fi
+	done
+
+	YNDEF=
+	return $RET
+}
+
 doexit()
 {
 	if [ $1 -eq 0 ]
@@ -107,44 +145,18 @@ then
 	ERRFILE=/dev/null
 fi
 
-$ECHO "\nConfiguring \"cddbd\" CDDB Protocol Server $CDDBD_VER by Steve Scherf"
+$ECHO "\nConfiguring \"cddbd\" CDDB Protocol Server $CDDBD_VER by Steve Scherf et al."
 
 # Determine BASEDIR.
 
 if [ -z "$BASEDIR" ]
 then
-	if [ -d /usr/lib/X11 ]
-	then
-		BASEDIR=/usr/lib/X11
-	elif [ -d /usr/X/lib ]
-	then
-		BASEDIR=/usr/X/lib
-	elif [ -d /usr/X11/lib/X11 ]
-	then
-		BASEDIR=/usr/X11/lib
-	elif [ -d /usr/X386/lib/X11 ]
-	then
-		BASEDIR=/usr/X386/lib
-	elif [ -d /usr/X11R5/lib/X11 ]
-	then
-		BASEDIR=/usr/X11R5/lib/X11
-	elif [ -d /usr/X11R6/lib/X11 ]
-	then
-		BASEDIR=/usr/X11R6/lib/X11
-	elif [ -d /usr/openwin/lib ]
-	then
-		BASEDIR=/usr/openwin/lib/X11
-	elif [ -d /usr/local/lib/X11 ]
-	then
-		BASEDIR=/usr/local/lib/X11
-	else
-		BASEDIR=/usr/lib/X11
-	fi
+    BASEDIR=/usr/local
 else
 	BASEDIR=`echo $BASEDIR | sed 's/\/\//\//g'`
 fi
 
-ACCESS=${BASEDIR}/xmcd/cddbd
+ACCESS=${BASEDIR}/cddbd
 
 while :
 do
@@ -169,10 +181,10 @@ echo
 
 umask $UMASK
 rm -f cddbd
-echo $ACCESS > .accessfile
+echo "ACCESSFILE $ACCESS" > .accessfile
 
 cat > access.h << __EOF__
-/* This file is generated by config.sh. */
+/* This file is generated by config.sh. Do not edit. */
 
 #ifdef _AIX
 #include <sys/select.h>
@@ -180,7 +192,84 @@ cat > access.h << __EOF__
 #endif
 
 #define ACCESSFILE "${ACCESS}/access"
+
 __EOF__
 
+cat > secure.c << __EOF__
+/* This file is generated by config.sh. Do not edit. */
+
+char *secure_users[] = {
+__EOF__
+
+$ECHO "Enter the list of trusted users, one per line."
+$ECHO "Press return alone when finished.\n"
+
+secusers=0
+
+while :
+do
+	if getstr "Secure user:"
+	then
+		if [ $ANS = "" ]
+		then
+			break
+		else
+			$ECHO "	\"$ANS\"," >> secure.c
+			secusers=`expr $secusers + 1`
+		fi
+	else
+		break
+	fi
+done
+
+cat >> secure.c << __EOF__
+	0
+};
+__EOF__
+
+echo ""
+
+echo "SECUSERS ${secusers}" >> .accessfile
+
+# Setup configurables.h
+rm -f ${CONFIGHEADER}
+
+echo "/* Generated by config.sh - do not edit */" >> ${CONFIGHEADER}
+echo "#ifndef __CONFIGURABLES_H__" >> ${CONFIGHEADER}
+echo "#define __CONFIGURABLES_H__" >> ${CONFIGHEADER}
+echo "" >> ${CONFIGHEADER}
+
+YNDEF=n
+getyn "\nDo you want to use Windows format of the DB"
+if [ $? -eq 0 ]
+then
+    echo "#define DB_WINDOWS_FORMAT" >> ${CONFIGHEADER}
+    YNDEF=y
+    getyn "\nDo you want to support DB files containing range of discid's start byte\n\
+- e.g. 01to33 (this is the standard format; otherwise there must be a file for\n\
+each discid's start byte - e.g. 01to01 (a little bit faster))"
+    if [ $? -eq 0 ]
+    then
+        echo "#define DB_WINDOWS_FORMAT_USE_RANGES" >> ${CONFIGHEADER}
+    else
+        echo "#undef DB_WINDOWS_FORMAT_USE_RANGES" >> ${CONFIGHEADER}
+    fi
+else
+    echo "#undef DB_WINDOWS_FORMAT" >> ${CONFIGHEADER}
+    echo "#undef DB_WINDOWS_FORMAT_USE_RANGES" >> ${CONFIGHEADER}
+fi
+
+YNDEF=n
+getyn "\nDo you want to disable peer address resolution (faster under Cygwin\n\
+when there is no DNS (no internet access))"
+if [ $? -eq 0 ]
+then
+    echo "#define DONT_RESOLVE_ADDRESS" >> ${CONFIGHEADER}
+else
+    echo "#undef DONT_RESOLVE_ADDRESS" >> ${CONFIGHEADER}
+fi
+
+echo "" >> ${CONFIGHEADER}
+echo "#endif /* __CONFIGURABLES_H__ */" >> ${CONFIGHEADER}
 
 doexit $ERROR
