@@ -1,9 +1,8 @@
 /*
  *   cddbd - CD Database Protocol Server
  *
- *   Copyright (C) 1996  Steve Scherf
- *   Email: steve@moonsoft.com
- *   Moondog Software Productions - makers of fine public domain software.
+ *   Copyright (C) 1996-1997  Steve Scherf (steve@moonsoft.com)
+ *   Portions Copyright (C) 1999-2006  by various authors
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,12 +16,12 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
 
 #ifndef LINT
-static char *_help_c_ident_ = "@(#)$Id: help.c,v 1.3 1996/12/13 23:35:22 steve Exp $";
+static char *const _help_c_ident_ = "@(#)$Id: help.c,v 1.9.2.1 2006/04/17 13:22:21 joerg78 Exp $";
 #endif
 
 
@@ -30,13 +29,15 @@ static char *_help_c_ident_ = "@(#)$Id: help.c,v 1.3 1996/12/13 23:35:22 steve E
 
 char *help_info[] = {
 	"",
-	"This is a cddb protocol (CDDBP) CD Database Server, used for",
-	"finding and retrieving cddb format Compact Disc data.",
+	"This is a CDDB protocol (CDDBP) CD Database Server, used for",
+	"finding and retrieving CDDB format Compact Disc data.",
 	"",
-	"Cddbd, the \"CD Database Server\" copyright (c) 1996 by Steve Scherf.",
-	"Cddb copyright (c) 1996 by Ti Kan.",
+	"Cddbd, the \"CD Database Server\" copyright (c) 1996-1997 by ",
+	"Steve Scherf. CDDB specification copyright (c) 1996-1997 by Ti Kan,",
+	"portions copyright (c) 1996-1997 by Steve Scherf.",
+	"Some modifications (c) 1999-2006 by various contributors.",
 	"",
-	"If you have questions or comments, send email to: steve@moonsoft.com.",
+	"If you have questions or comments, send email to: info@freedb.org.",
 	0
 };
 
@@ -51,7 +52,7 @@ char *help_help[] = {
 };
 
 char *cddb_help[] = {
-	"CDDB <subcmd> (valid subcmds: HELLO QUERY READ SRCH WRITE)",
+	"CDDB <subcmd> (valid subcmds: HELLO LSCAT QUERY READ UNLINK WRITE)",
 	"    Performs a CD database operation.",
 	"    Arguments are:",
 	"        subcmd:  CDDB subcommand to print help for.",
@@ -62,20 +63,23 @@ char *cddb_help[] = {
 
 char *get_help[] = {
 	"GET <file>",
-	"    Gets a server system file, or lists a directory.",
+	"    Gets a server system file.",
 	0
 };
 
 char *log_help[] = {
-	"LOG [[start_time [end_time]] | [day [#_of_days]] | [get]]",
+	"LOG [-l lines] [get [-f flag]] [start_time [end_time]] | [day [days]]",
 	"    Shows message log statistics.",
 	"    Arguments are:",
+	"        lines:       max number of lines of list info to print.",
+	"        get:         the string \"get\", to return the log itself.",
+	"        flag:        the flag to filter messages by.",
 	"        start_time:  date to count messages after.",
 	"        end_time:    date to count messages up to.",
 	"                     Format: hh[mm[ss[MM[DD[YY]]]]]",
-	"        day:         The string \"day\", to count messages generated",
-	"                     within the last #_of_days (default 1 day).",
-	"        get:         The string \"get\", to return the log itself.",
+	"        day:         the string \"day\", to count messages generated",
+	"                     within the specified # of days (default 1 day).",
+	"        days:        the number of days worth of messages to count.",
 	0
 };
 
@@ -94,12 +98,24 @@ char *proto_help[] = {
 	0
 };
 
+char *put_help[] = {
+	"PUT <file>",
+	"    Puts a server system file.",
+	0
+};
+
 char *quit_help[] = {
 	"QUIT",
 	"    Close database server connection.",
 	0
 };
 
+char *validate_help[] = {
+	"VALIDATE",
+	"    Perform user validation.",
+	0
+};
+
 char *ver_help[] = {
 	"VER",
 	"    Print cddbd version information.",
@@ -152,6 +168,7 @@ char *query_help[] = {
 	0
 };
 
+/*
 char *srch_help[] = {
 	"    SRCH  <keyword> <search_type> ... <search_type>",
 	"        Perform a keyword search.",
@@ -167,6 +184,7 @@ char *srch_help[] = {
 	"                trk:      signifies a CD track title search.",
 	0
 };
+*/
 
 char *read_help[] = {
 	"    READ  <category> <discid>",
@@ -185,3 +203,30 @@ char *write_help[] = {
 	"            discid:    CD disk ID number.",
 	0
 };
+
+char *lscat_help[] = {
+	"    LSCAT",
+	"        List all database categories.",
+	0
+};
+
+char *discid_help[] = {
+	"DISCID <ntrks> <off_1> <off_2> <...> <off_n> <nsecs>",
+	"    Calculate a CDDB disc ID for a given set of CD TOC information.",
+	"    Arguments are:",
+	"        ntrks:  total number of tracks on CD.",
+	"        off_X:  frame offset of track X.",
+	"        nsecs:  total playing length of the CD in seconds.",
+	0
+};
+
+
+char *unlink_help[] = {
+	"    UNLINK <category> <discid>",
+	"        Delete a database entry or link.",
+	"        Arguments are:",
+	"            category:  CD category.",
+	"            discid:    CDDB disc ID to be deleted.",
+	0
+};
+
