@@ -1,9 +1,8 @@
 /*
  *   cddbd - CD Database Protocol Server
  *
- *   Copyright (C) 1996-1997  Steve Scherf
- *   Email: steve@moonsoft.com
- *   Moondog Software Productions - makers of fine public domain software.
+ *   Copyright (C) 1996-1997  Steve Scherf (steve@moonsoft.com)
+ *   Portions Copyright (C) 2001-2003  by various authors
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,16 +16,15 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
 
 #ifndef LINT
-static char *_inet_c_ident_ = "@(#)$Id: inet.c,v 1.11 2001/09/11 23:21:34 joerg78 Exp $";
+static char *const _inet_c_ident_ = "@(#)$Id: inet.c,v 1.26 2004/08/30 19:46:19 megari Exp $";
 #endif
 
 #include <stdlib.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <ctype.h>
@@ -42,10 +40,8 @@ static char *_inet_c_ident_ = "@(#)$Id: inet.c,v 1.11 2001/09/11 23:21:34 joerg7
 #include <arpa/inet.h>
 #include <time.h>
 #include <sys/time.h>
-#include <stdio.h>
 #include "patchlevel.h"
 #include "access.h"
-#include "list.h"
 #include "cddbd.h"
 
 #ifndef INADDR_NONE
@@ -132,8 +128,12 @@ get_rmt_hostname(int fd, char *thost, char *rh)
 	socklen_t len;
 	struct hostent *hp;
 	struct sockaddr_in name;
+#ifdef DONT_RESOLVE_ADDRESS
+	struct hostent he;
+#endif
 
 	len = (socklen_t)sizeof(name);
+	memset(&name, 0, len); /* need to be sure it is "initialised" */
 
 	switch(interface) {
 	case IF_HTTP:
@@ -144,8 +144,13 @@ get_rmt_hostname(int fd, char *thost, char *rh)
 		}
 		else {
 			sinfo.si_haddr.s_addr = inet_addr(a);
+#ifdef DONT_RESOLVE_ADDRESS
+			he.h_name = a;
+			hp = &he;
+#else
 			hp = gethostbyaddr((char *)&sinfo.si_haddr,
 			    sizeof(sinfo.si_haddr), AF_INET);
+#endif
 		}
 
 		break;
@@ -200,11 +205,15 @@ match_host(char *ahost)
 {
 	char *p;
 	char **ap;
-	char net[CDDBBUFSIZ];
 	unsigned long in;
-	struct netent *np;
 	struct hostent *hp;
-	struct in_addr neta;
+
+	#ifndef __CYGWIN32__
+	char net[CDDBBUFSIZ];
+	struct netent *np;	
+	struct in_addr neta;	
+	#endif /* __CYGWIN32__ */
+
 
 	/* Try and match the default definition. */
 	if(!strcmp(ahost, "default")) {
@@ -477,7 +486,7 @@ cddbd_stand(int port)
 		signal(SIGHUP, SIG_IGN);
 
 		if(sock < 0) {
-			if(errno == EINTR)
+			if(errno == EINTR || errno == ECHILD) /* "No children" is no real error state! */
 				continue;
 
 			perror("accept");
@@ -782,13 +791,76 @@ cddbp_log_stats(void)
 	return 0;
 }
 
+/* determines the highest common protocol level between local and remote     */
+/* and sets the protocol level on the remote accordingly. Returns that level */
+int
+cddbp_setproto(void) {
+	char buf[CDDBBUFSIZ];
+	char errstr[CDDBBUFSIZ];
+	int max_send_proto; /* highest protocol level the remote server understands */
+	int use_proto; /* protocol level to be used for transmission */
+
+	/* let's find out the highest protocol level the remote server supports */
+	if(!sock_write("proto\n", 0, 1)) {
+		cddbd_log(LOG_ERR | LOG_NET,
+		    "Failed to send proto command to %s server.", cddbphost);
+		sock_close();
+		return 0;
+	}
+
+	if(!sock_serv_read(buf, sizeof(buf), 200, errstr)) {
+		cddbd_log(LOG_ERR | LOG_NET,
+			"Failed to send proto command to %s server: %s.",cddbphost, errstr);
+		sock_close();
+		return 0;
+	}
+	
+	if(sscanf(buf, "200 CDDB protocol level: current %*d, supported %d", &max_send_proto) != 1) {
+			cddbd_log(LOG_ERR | LOG_NET,
+			"Malformed response to proto command: %s",
+			buf);
+		sock_close();
+		return 0;
+	}
+	
+	/***************************************************************/
+	/* highest protocol level of remote determined - compare to    */
+	/* the hightest level we support and determine highest common  */
+	/* protocol level                                              */
+	/***************************************************************/
+	if (MAX_PROTO_LEVEL > max_send_proto)
+		use_proto=max_send_proto;
+	else
+		use_proto=MAX_PROTO_LEVEL;
+		
+	/* set the protocol level on remote server */
+	
+	cddbd_snprintf(buf, sizeof(buf), "proto %d\n", use_proto);
+	if(!sock_write(buf, 0, 1)) {
+		cddbd_log(LOG_ERR | LOG_NET,
+		    "Failed to set protocol level on %s server.",cddbphost);
+		sock_close();
+		return 0;
+	}
+	
+	if(!sock_serv_read(buf, sizeof(buf), 201, errstr)) {
+		cddbd_log(LOG_ERR | LOG_NET,
+			"Failed to set protocol level on %s server: %s.",cddbphost, errstr);
+		sock_close();
+		return 0;
+	}
+	return use_proto;
+}
 
 int
-cddbp_transmit(db_t *db, char *category, unsigned int discid)
+cddbp_transmit(db_t *db, char *category, unsigned int discid, int use_proto)
 {
 	char buf[CDDBBUFSIZ];
 	char errstr[CDDBBUFSIZ];
-
+	
+	/* convert charset of entry to be transmitted according to used protocol level */
+	convert_db_charset_proto(db,use_proto);
+	
 	cddbd_snprintf(buf, sizeof(buf), "cddb write %s %08x\n",
 	    category, discid);
 
@@ -806,7 +878,11 @@ cddbp_transmit(db_t *db, char *category, unsigned int discid)
 		return 0;
 	}
 
-	if(!db_write(sock_fp, db, MAX_PROTO_LEVEL)) {
+#ifdef DB_WINDOWS_FORMAT
+	if(!db_write(sock_fp, db, use_proto, 0)) {
+#else
+	if(!db_write(sock_fp, db, use_proto)) {
+#endif
 		cddbd_log(LOG_ERR | LOG_XMIT,
 		    "Failed to write DB entry to %s server (%d).",
 		    cddbphost, errno);
@@ -1076,7 +1152,7 @@ smtp_command(csite_t *sp, char *rcmd, int lev, char *hto, char *mfrom)
 	cddbd_snprintf(buf2, sizeof(buf2), "cmd_%08x", cddbd_rand());
 	cddbd_snprintf(buf, sizeof(buf), cmd_send_subj2, buf2);
 
-	if(!smtp_transmit(fp, buf, hto, hto, mfrom, 0, MF_MULTI, 0, key)) {
+	if(!smtp_transmit(fp, -1, buf, hto, hto, mfrom, 0, MF_MULTI, 0, key)) {
 		cddbd_log(LOG_ERR | LOG_XMIT,
 		    "Failed to send command \"%s\" to %s server (%d)",
 		    rcmd, smtphost, errno);
@@ -1094,7 +1170,7 @@ smtp_command(csite_t *sp, char *rcmd, int lev, char *hto, char *mfrom)
 
 
 int
-smtp_transmit(FILE *fp, char *subject, char *rcpt, char *hto, char *mfrom,
+smtp_transmit(FILE *fp, int set, char *subject, char *rcpt, char *hto, char *mfrom,
     char *echo, int flags, int len, ct_key_t *key)
 {
 	char *crcstr;
@@ -1208,7 +1284,7 @@ smtp_transmit(FILE *fp, char *subject, char *rcpt, char *hto, char *mfrom,
 		cddbd_snprintf(bound, sizeof(bound), "---=====cddbd_%08X%08X_",
 		    cddbd_rand(), cddbd_rand());
 
-		if(!smtp_write_hdr_type(multi_alt, bound, CC_ISO_8859_1))
+		if(!smtp_write_hdr_type(multi_alt, bound, set))
 			return 0;
 
 		/* Separate the header from the body. */
@@ -1222,20 +1298,20 @@ smtp_transmit(FILE *fp, char *subject, char *rcpt, char *hto, char *mfrom,
 		if(!smtp_write_bound(bound, 0))
 			return 0;
 
-		if(!smtp_write_body(fp, CE_BASE64, CC_ISO_8859_1, crcstr))
+		if(!smtp_write_body(fp, CE_BASE64, set, crcstr))
 			return 0;
 
 		if(!smtp_write_bound(bound, 0))
 			return 0;
 
-		if(!smtp_write_body(fp, CE_QUOTED_PRINT, CC_ISO_8859_1, crcstr))
+		if(!smtp_write_body(fp, CE_QUOTED_PRINT, set, crcstr))
 			return 0;
 
 		if(!smtp_write_bound(bound, 1))
 			return 0;
 	}
 	else if(flags & MF_ENC) {
-		if(!smtp_write_body(fp, CE_QUOTED_PRINT, CC_ISO_8859_1, crcstr))
+		if(!smtp_write_body(fp, CE_QUOTED_PRINT, set, crcstr))
 			return 0;
 	}
 	else {
@@ -1504,10 +1580,11 @@ int
 smtp_write_hdr_type(char *type, char *bound, int set)
 {
 	char buf[CDDBBUFSIZ];
+	char *setname = (set == -1) ? "x-unknown" : charsets[set].name;
 
 	if(bound != 0) {
 		cddbd_snprintf(buf, sizeof(buf), "%s: %s; %s=%s;\n",
-		    content_type, type, charset, char_types[set]);
+		    content_type, type, charset, setname);
 
 		if(!smtp_write_hdr(buf))
 			return 0;
@@ -1517,7 +1594,7 @@ smtp_write_hdr_type(char *type, char *bound, int set)
 	}
 	else {
 		cddbd_snprintf(buf, sizeof(buf), "%s: %s; %s=%s\n",
-		    content_type, type, charset, char_types[set]);
+		    content_type, type, charset, setname);
 	}
 
 	return(smtp_write_hdr(buf));
@@ -1542,8 +1619,8 @@ sock_open(char *shost, int ind, int rport, char *errstr)
 	}
 	else {
 		/* T2D: inet_addr() should be replaced by inet_aton() which
-                 *      allows a cleaner error handling.
-                 */
+		 * allows a cleaner error handling.
+		 */
 		if(isipaddr(shost) && (ad.s_addr = inet_addr(shost)) != INADDR_NONE) {
 			cddbd_bcopy((char *)&ad.s_addr, (char *)&sin.sin_addr,
 			    sizeof(ad.s_addr));
@@ -1596,7 +1673,17 @@ sock_close(void)
 	sock_fd = -1;
 }
 
-
+/*********************************************************/
+/* reads a server response from socket and checks if     */
+/* the response code matches the expected code           */
+/*-------------------------------------------------------*/
+/* buf: array to be read to                              */
+/* size: maximum number of bytes to be read from socket  */
+/* expect: expected response code                        */
+/* errstr: array to be filled with error message, if any */
+/* returns: 1 on success (successful read                */
+/*          & expected matched), 0 on error              */
+/*-------------------------------------------------------*/
 int
 sock_serv_read(char *buf, int size, int expect, char *errstr)
 {
@@ -1623,11 +1710,19 @@ sock_serv_read(char *buf, int size, int expect, char *errstr)
 	return 0;
 }
 
-
+/******************************************************/
+/* writes a string to a socket                        */
+/*----------------------------------------------------*/
+/* buf: string to be written to the socket            */
+/* cnt: maximum number of bytes to be written         */
+/*      if cnt==0, then cnt is set to length of buf   */
+/* flag: specifies, whether sent data shall be logged */
+/* returns: 1 on success, 0 on error                  */
+/*----------------------------------------------------*/
 int
 sock_write(char *buf, int cnt, int flag)
 {
-	int cc;
+	int cc; /* number of bytes written to socket, -1 on error */
 	char *p;
 	char *pbuf;
 
@@ -1636,15 +1731,17 @@ sock_write(char *buf, int cnt, int flag)
 
 	p = buf;
 
+	/* write data to socket */
 	while(cnt > 0) {
 		cc = write(sock_fd, p, cnt);
-		if(cc < 0)
-			return 0;
+		if(cc < 0) /* an error occurred writing to socket */
+			return 0; 
 
 		p += cc;
 		cnt -= cc;
 	}
 
+	/* if in verbose mode and flag for logging set, log sent data */
 	if(verbose && flag) {
 		pbuf = strdup(buf);
 
@@ -1661,6 +1758,14 @@ sock_write(char *buf, int cnt, int flag)
 }
 
 
+/*******************************************************/
+/* reads a string from socket                          */
+/*-----------------------------------------------------*/
+/* buf: array to be read to                            */
+/* cnt: maximum number of bytes to be read from socket */
+/* flag: specifies, whether read data shall be logged  */
+/* returns: 1 on success, 0 on error                   */
+/*-----------------------------------------------------*/
 int
 sock_read(char *buf, int cnt, int flag)
 {
@@ -1678,7 +1783,8 @@ sock_read(char *buf, int cnt, int flag)
 
 		if(c == '\n' || cnt == 1) {
 			*buf = '\0';
-
+			
+			/* if in verbose mode and flag for logging set, log read data */
 			if(verbose && flag) {
 				pbuf = strdup(p);
 
