@@ -1,12 +1,11 @@
 #!/bin/sh
 #
-#   @(#)$Id: install.sh,v 1.5 2001/06/30 12:50:32 joerg78 Exp $
+#   @(#)$Id: install.sh,v 1.17.2.6 2006/07/01 17:38:10 megari Exp $
 #
 #   cddbd - CD Database Protocol Server
 #
-#   Copyright (C) 1996  Steve Scherf
-#   Email: steve@moonsoft.com
-#   Moondog Software Productions - makers of fine public domain software.
+#   Copyright (C) 1996  Steve Scherf (steve@moonsoft.com)
+#   Portions Copyright (C) 2001-2006  by various authors 
 #
 #   Based on the original source by Ti Kan.
 #
@@ -22,16 +21,26 @@
 #
 #   You should have received a copy of the GNU General Public License
 #   along with this program; if not, write to the Free Software
-#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
 PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc:/usr/local/bin:/usr/5bin
 export PATH
 
-CDDBD_VER=1.4
+CDDBD_VER=1.5.2
 ERRFILE=/tmp/cddbd.err
 TMPFILE=/tmp/cddbd.$$
 
+if [ -x cddbd.exe ]
+then
+    CDDBDEXEC=cddbd.exe
+else
+    CDDBDEXEC=cddbd
+fi
+SUBMITCGI=submit.cgi
+
+MANPAGE=cddbd.1
+
 id=`id`
 
 # Set up access defaults.
@@ -40,6 +49,8 @@ FILEPERM=644
 PWDPERM=600
 DBINPERM1=700
 DBINPERM2=6711
+DCDDBCGIPERM=6711
+SUBMITCGIPERM=711
 UMASK=022
 OWNER=`expr "$id" : '.*uid=.*(\(.*\)).*gid'`
 GROUP=`expr "$id" : '.*gid=[0-9]*(\([a-zA-Z0-9_]*\))'`
@@ -49,9 +60,10 @@ HANGS=10
 LINES=1024
 SIZE=102400
 XMITS=4
-HIWAT=1024000
-LOWAT=768000
+HIWAT=10240000
+LOWAT=7680000
 STRIP=no
+BRZRK=yes
 FUZZY=900
 FUZDV=4
 XTIME=600
@@ -64,12 +76,16 @@ MTIME=500
 HTIME=60
 LTIME=250
 LWAIT=60
-LOGFLAGS="errors hello access info"
+LOGFLAGS="errors hello access"
+DHOSTNAME="`uname -n`"
 DSMTPHOST="localhost"
 DEMAILADDR="${OWNER}@`uname -n`"
 DFEEDHOST="*.freedb.org"
 ADMINMAIL="info@freedb.org"
-
+CHARSET="prefer_iso"
+UTFASISO="reject"
+DSUBMITADDR="freedb-submit@freedb.org"
+DTESTSUBMITADDR="test-submit@freedb.org"
 
 #
 # Utility functions
@@ -79,7 +95,7 @@ doexit()
 {
 	if [ $1 -eq 0 ]
 	then
-		$ECHO "\nInstallation of cddbd is now complete."
+		$ECHO "\nInstallation of ${CDDBDEXEC} is now complete."
 	else
 		$ECHO "\nErrors have occurred in the installation."
 		if [ $ERRFILE != /dev/null ]
@@ -269,6 +285,17 @@ else
 	ECHO=echo					# generic SysV
 fi
 
+if [ -x /usr/sbin/sendmail ]
+then
+	DSENDMAIL=/usr/sbin/sendmail
+elif [ -x /usr/lib/sendmail ]
+then
+	DSENDMAIL=/usr/sbin/sendmail
+else
+	DSENDMAIL=""
+fi
+
+
 # Remove old error log file
 ERROR=0
 rm -f $ERRFILE
@@ -284,7 +311,7 @@ then
 	doexit 1
 fi
 
-if [ ! -x cddbd ]
+if [ ! -x ${CDDBDEXEC} ]
 then
 	$ECHO "You must make cddbd before installing."
 	doexit 1
@@ -307,7 +334,7 @@ then
 fi
 
 
-$ECHO "\nInstalling \"cddbd\" CDDB Protocol Server $CDDBD_VER by Steve Scherf et al."
+$ECHO "\nInstalling \"${CDDBDEXEC}\" CDDB Protocol Server $CDDBD_VER by Steve Scherf et al."
 
 
 # Determine BINDIR
@@ -330,6 +357,9 @@ else
 	BINDIR=`echo $BINDIR | sed 's/\/\//\//g'`
 fi
 
+# get the man path
+MANDIR=`echo $BINDIR | sed 's/bin$/man\/man1/g'`
+
 while :
 do
 	if getstr "\nEnter cddbd binary directory\n[${BINDIR}]:"
@@ -346,6 +376,22 @@ do
 	fi
 done
 
+while :
+do
+	if getstr "\nEnter cddbd man directory (for cddbd.1)\n[${MANDIR}]:"
+	then
+		if [ -d "$ANS" ]
+		then
+			MANDIR=$ANS
+			break
+		else
+			$ECHO "Error: $ANS does not exist."
+		fi
+	else
+		break
+	fi
+done
+
 
 # Find other directories
 
@@ -381,8 +427,15 @@ do
 	then
 		if [ -d "$ANS" ]
 		then
-			CDDBDIR=$ANS
-			break
+			if [ "$ANS" = "$WORKDIR" ]
+			then
+				$ECHO "The database directory cannot be the \c"
+				$ECHO "same as the work directory."
+			else
+				CDDBDIR=$ANS
+				break
+			fi
+
 		else
 			CDDBTMP=$ANS
 			YNDEF=n
@@ -400,7 +453,12 @@ done
 YNDEF=y
 if getyn "\nDo you want to enable server access via HTTP"
 then
-	CGIDIR="$HOME/public_html"
+	if [ -d "$HOME/public_html" ]
+	then
+		CGIDIR="$HOME/public_html"
+	else
+		CGIDIR=""
+	fi
 
 	while :
 	do
@@ -409,40 +467,78 @@ then
 			if [ -d "$ANS" ]
 			then
 				CGIDIR=$ANS
-				$ECHO ""
 				break
 			else
 				$ECHO "Error: $ANS does not exist."
 			fi
 		else
-			$ECHO ""
-			break
+			if [ "$CGIDIR" != "" ]
+			then
+				break
+			else
+				$ECHO: "Error: a directory must be entered."
+			fi
 		fi
 	done
+	$ECHO "\nSubmissions via http are handled by the submit.cgi. The submit.cgi"
+	$ECHO "forwards the submissions it receives to the central submit e-mail"
+	$ECHO "address. To do that, it needs sendmail (or a sendmail wrapper) installed."
+	if getyn "\nDo you want to accept submissions via http"
+	then
+		instsubmitcgi=true
+		rm -f submit.cgi
+		cat ${SUBMITCGI}.template > ${SUBMITCGI}
+		while :
+		do
+			if getstr "\nEnter the path to sendmail.\n[${DSENDMAIL}]:"
+			then
+				if [ -x "$ANS" -a -f "$ANS" ]
+				then
+					ANS=`echo $ANS | sed 's/\//\\\\\//g'`
+					sed "s/\/usr\/sbin\/sendmail/$ANS/" ${SUBMITCGI} > ${SUBMITCGI}.tmp && mv ${SUBMITCGI}.tmp ${SUBMITCGI}
+					break
+				else
+					$ECHO "Error: $ANS does not exist."
+				fi
+			else
+				if [ "$DSENDMAIL" != "" ]
+				then
+					DSENDMAIL=`$ECHO $DSENDMAIL | sed 's/\//\\\\\//'`
+					sed "s/\/usr\/sbin\/sendmail/$DSENDMAIL/" ${SUBMITCGI} > ${SUBMITCGI}.tmp && mv ${SUBMITCGI}.tmp ${SUBMITCGI}
+					break
+				else
+					$ECHO "The path to sendmail must be entered."
+				fi
+			fi
+		done
+		if getstr "\nEnter submit e-mail address.\n[${DSUBMITADDR}]:"
+		then
+			sed "s/${DSUBMITADDR}/$ANS/" ${SUBMITCGI} > ${SUBMITCGI}.tmp && mv ${SUBMITCGI}.tmp ${SUBMITCGI}
+		fi
+		if getstr "\nEnter test-submit e-mail address.\n[${DTESTSUBMITADDR}]:"
+		then
+			sed "s/${DTESTSUBMITADDR}/$ANS/" ${SUBMITCGI} > ${SUBMITCGI}.tmp && mv ${SUBMITCGI}.tmp ${SUBMITCGI}
+		fi
+	fi
 fi
 
 # Make directory names.
 
 SERVDIR="${WORKDIR}/server"
-HASHDIR="${WORKDIR}/hash"
 
 # Make file names.
 
 ACCESS="${BASEDIR}/access"
+ALTACC="${WORKDIR}/access.alt"
 MOTD="${WORKDIR}/motd"
 SITE="${WORKDIR}/sites"
 PASSWD="${WORKDIR}/passwd"
-HISTORY="${WORKDIR}/history"
-NHISTORY="${SERVDIR}/history"
-FUZZFILE="${HASHDIR}/fuzzy_index"
-NFUZZFILE="${SERVDIR}/fuzzy_index"
-LOG="${WORKDIR}/log"
-NLOG="${SERVDIR}/log"
 
 # Create the access file.
 
 umask $UMASK
 rm -f access
+rm -f access.alt
 cp access.hdr access
 chmod $FILEPERM access
 
@@ -458,14 +554,6 @@ rm -f passwd
 cat passwd.hdr > passwd
 chmod $PWDPERM passwd
 
-YNDEF=y
-if getyn "Do you want to enable logging of server accesses"
-then
-	logging=norm
-else
-	logging=none
-fi
-
 $ECHO "motdfile:     ${MOTD}" >> access
 $ECHO "sitefile:     ${SITE}" >> access
 $ECHO "pwdfile:      ${PASSWD}" >> access
@@ -473,7 +561,7 @@ $ECHO "workdir:      ${WORKDIR}" >> access
 $ECHO "cddbdir:      ${CDDBDIR}" >> access
 
 YNDEF=n
-if getyn "Do you want configure your site as a master database hub"
+if getyn "\nDo you want configure your site as a master database hub"
 then
 	master=true
 	logging=post
@@ -506,11 +594,18 @@ then
 	$ECHO "bounce_email: ${EMAILADDR}" >> access
 	$ECHO "test_email:   ${EMAILADDR}" >> access
 else
+	logging=norm
 	master=false
 	$ECHO "postdir:      ${CDDBDIR}" >> access
 	$ECHO "dup_policy:   always" >> access
 fi
 
+if getstr "\nEnter the hostname used for banner message etc.\n[${DHOSTNAME}]:"
+then
+	$ECHO "apphost:      $ANS" >> access
+fi
+
+
 case $logging in
 	norm)
 		$ECHO "logging:      ${LOGFLAGS}" >> access
@@ -542,6 +637,9 @@ $ECHO "max_hangs:    ${HANGS}" >> access
 $ECHO "log_hiwat:    ${HIWAT}" >> access
 $ECHO "log_lowat:    ${LOWAT}" >> access
 $ECHO "strip_ext:    ${STRIP}" >> access
+$ECHO "ck_berzerk:   ${BRZRK}" >> access
+$ECHO "file_charset: ${CHARSET}" >> access
+$ECHO "utf_as_iso:   ${UTFASISO}" >> access
 
 $ECHO "\nEnter user and permission information. In general, you will want"
 $ECHO "the server and database user/group/perms to be the same. They MUST"
@@ -619,6 +717,22 @@ then
 	BINPERM=$ANS
 fi
 
+if [ "$CGIDIR" != "" ]
+then
+	$ECHO "\nThe cddb.cgi file binary mode required depends on your web server settings."
+	$ECHO "The cddb.cgi should be executed as the user who owns the server directory."
+	$ECHO "If you are installing cddb.cgi in the public_html directory of that user"
+	$ECHO "and executing the cgi suid is already taken care of by Apache suexec, then"
+	$ECHO "you should specify 711 here, otherwise leave the default of 6711."
+	if getstr "\nEnter the cddb.cgi binary mode\n[${DCDDBCGIPERM}]:"
+	then
+		CDDBCGIPERM=$ANS
+	else
+		CDDBCGIPERM=$DCDDBCGIPERM
+	fi
+fi
+
+
 $ECHO "user:         ${OWNID}" >> access
 $ECHO "group:        ${GRPID}" >> access
 $ECHO "file_mode:    ${FPERM}" >> access
@@ -628,44 +742,64 @@ $ECHO "db_group:     ${DBGRPID}" >> access
 $ECHO "db_file_mode: ${DBFPERM}" >> access
 $ECHO "db_dir_mode:  ${DBDPERM}" >> access
 
-$ECHO "\n# Beginning of permissions.\n" >> access
+PERMACC=access
 
-YNDEF=n
-if getyn "\nDo you want to allow automatic database updates"
-then
-	if getstr "\nEnter database feed site\n[${DFEEDHOST}]:"
-	then
-		FEEDHOST=$ANS
-	else
-		FEEDHOST=$DFEEDHOST
-	fi
-
-	$ECHO "host_perms: sc ${FEEDHOST} connect post update \c" >> access
-	$ECHO "get put nopasswd" >> access
-
-	$ECHO "\nIn order for automatic updates to be initiated, you must"
-	$ECHO "contact the cddbd administrator at \c"
-
-	if [ "${FEEDHOST}" = "${DFEEDHOST}" ]
-	then
-		$ECHO "${ADMINMAIL}.\n"
-	else
-		$ECHO "the site you have specified.\n"
-	fi
-fi
 if [ $master = true ]
 then
+	$ECHO "\n# Beginning of permissions.\n" >> access
 	$ECHO "host_perms: ches default connect nopost noupdate noget \c" >> access
 	$ECHO "noput nopasswd" >> access
 else
-	$ECHO "host_perms: che default connect nopost noupdate noget \c" >> access
-	$ECHO "noput nopasswd" >> access
-	$ECHO "host_perms: s default noconnect nopost noupdate noget \c" >> access
-	$ECHO "noput nopasswd" >> access
+	YNDEF=n
+	if getyn "\nDo you want to allow automatic database updates"
+	then
+		YNDEF=n
+		if getyn "\nDo you want your server remotely administered"
+		then
+			$ECHO "altaccfile:   ${ALTACC}" >> access
+
+			PERMACC=access.alt
+			PUT=put
+			touch access.alt
+			chmod $FILEPERM access.alt
+			$ECHO "\n# Beginning of permissions.\n" >> access
+		else
+			PUT=noput
+		fi
+
+		$ECHO "\n# Beginning of permissions.\n" >> $PERMACC
+
+		if getstr "\nEnter database feed site\n[${DFEEDHOST}]:"
+		then
+			FEEDHOST=$ANS
+		else
+			FEEDHOST=$DFEEDHOST
+		fi
+
+		$ECHO "host_perms: sc ${FEEDHOST} connect post update \c" >> access
+		$ECHO "get $PUT nopasswd" >> access
+
+		$ECHO "\nIn order for automatic updates to be initiated, you must"
+		$ECHO "contact the cddbd administrator at \c"
+
+		if [ "${FEEDHOST}" = "${DFEEDHOST}" ]
+		then
+			$ECHO "${ADMINMAIL}.\n"
+		else
+			$ECHO "the site you have specified.\n"
+		fi
+	else
+		$ECHO "\n# Beginning of permissions.\n" >> access
+	fi
+
+	$ECHO "host_perms: che default connect nopost noupdate noget \c" >> $PERMACC
+	$ECHO "noput nopasswd" >> $PERMACC
+	$ECHO "host_perms: s default noconnect nopost noupdate noget \c" >> $PERMACC
+	$ECHO "noput nopasswd" >> $PERMACC
 fi
 
 YNDEF=y
-if getyn "Do you want to create the motd file now"
+if getyn "\nDo you want to create the motd file now"
 then
 	if [ "$EDITOR" = "" ]
 	then
@@ -702,42 +836,6 @@ makedir $SERVDIR $DPERM $OWNID $GRPID
 
 $ECHO "\nPreserving old files..."
 
-if [ -f $LOG ]
-then
-	$ECHO "\t$LOG -> $NLOG"
-	mv $LOG $NLOG
-
-	err=$?
-	if [ $err -ne 0 ]
-	then
-		logerr -n "Failed to preserve $LOG."
-	fi
-fi
-
-if [ -f $HISTORY ]
-then
-	$ECHO "\t$HISTORY -> $NHISTORY"
-	mv $HISTORY $NHISTORY
-
-	err=$?
-	if [ $err -ne 0 ]
-	then
-		logerr -n "Failed to preserve $HISTORY."
-	fi
-fi
-
-if [ -f $FUZZFILE ]
-then
-	$ECHO "\t$FUZZFILE -> $NFUZZFILE"
-	mv $FUZZFILE $NFUZZFILE
-
-	err=$?
-	if [ $err -ne 0 ]
-	then
-		logerr -n "Failed to preserve $FUZZFILE."
-	fi
-fi
-
 if [ -f $ACCESS ]
 then
 	$ECHO "\t$ACCESS -> ${ACCESS}.old"
@@ -746,7 +844,19 @@ then
 	err=$?
 	if [ $err -ne 0 ]
 	then
-		logerr -n "Failed to preserve $SITE."
+		logerr -n "Failed to preserve $ACCESS."
+	fi
+fi
+
+if [ -f $ALTACC -a -f access.alt ]
+then
+	$ECHO "\t$ALTACC -> ${ALTACC}.old"
+	mv $ALTACC ${ALTACC}.old
+
+	err=$?
+	if [ $err -ne 0 ]
+	then
+		logerr -n "Failed to preserve $ALTACC."
 	fi
 fi
 
@@ -794,11 +904,24 @@ fi
 $ECHO "\nInstalling files..."
 
 # Binaries
-instfile cddbd ${BINDIR}/cddbd $BINPERM $OWNID $GRPID
+instfile ${CDDBDEXEC} ${BINDIR}/${CDDBDEXEC} $BINPERM $OWNID $GRPID
+
+if [ -f access.alt ]
+then
+	instfile access.alt $ALTACC $FPERM $OWNID $GRPID
+fi
+
+# Man pages
+mkdir -p $MANDIR
+instfile ${MANPAGE} ${MANDIR}/${MANPAGE} $FILEPERM $OWNID $GRPID
 
 if [ "$CGIDIR" != "" ]
 then
-	instfile cddbd ${CGIDIR}/cddb.cgi $BINPERM $OWNID $GRPID
+	instfile ${CDDBDEXEC} ${CGIDIR}/cddb.cgi $CDDBCGIPERM $OWNID $GRPID
+	if [ $instsubmitcgi = true ]
+	then
+		instfile ${SUBMITCGI} ${CGIDIR}/submit.cgi $SUBMITCGIPERM $OWNID $GRPID
+	fi
 fi
 
 # Configuration files
@@ -811,19 +934,13 @@ then
 	instfile motd $MOTD $FPERM $OWNID $GRPID
 fi
 
-# Remove old directories.
-
-$ECHO "\nRemoving old directories..."
-
-removedir $HASHDIR
-
 YNDEF=y
 if getyn "\nDo you want to create the fuzzy matching hash file now"
 then
 	$ECHO "\nPlease wait while the hash file is created."
 	$ECHO "This will take a few minutes.\n"
 
-	${BINDIR}/cddbd -fd
+	${BINDIR}/${CDDBDEXEC} -fd
 
 	err=$?
 	if [ $err -ne 0 ]
@@ -832,7 +949,7 @@ then
 	fi
 else
 	$ECHO "\nIf you wish to enable fuzzy matching in the future, you will"
-	$ECHO "have to run \"cddbd -fd\"."
+	$ECHO "have to run \"${CDDBDEXEC} -fd\"."
 fi
 
 doexit $ERROR
