@@ -1,11 +1,8 @@
 /*
  *   cddbd - CD Database Protocol Server
  *
- *   Copyright (C) 1996-1997  Steve Scherf
- *   Email: steve@moonsoft.com
- *   Moondog Software Productions - makers of fine public domain software.
- *
- *   07.03.2000 modified to comply protocol version 4 -OGF-
+ *   Copyright (C) 1996-1997  Steve Scherf (steve@moonsoft.com)
+ *   Portions Copyright (C) 1999-2006  by various authors
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,13 +16,17 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
 
+#ifndef __PATCHLEVEL_H__
+#define __PATCHLEVEL_H__
+
 #ifndef LINT
-static char *_patchlevel_h_ident_ = "@(#)$Id: patchlevel.h,v 1.9 2001/09/10 15:33:41 joerg78 Exp $";
+static char *const _patchlevel_h_ident_ = "@(#)$Id: patchlevel.h,v 1.19.2.4 2006/04/14 14:45:06 joerg78 Exp $";
 #endif
 
-#define VERSION		"1.4"	/* The version number. */
+#define VERSION		"1.5.2"	/* The version number. */
 #define PATCHLEVEL	0		/* The patch level. */
+#endif
