@@ -1,9 +1,8 @@
 /*
  *   cddbd - CD Database Protocol Server
  *
- *   Copyright (C) 1996-1997  Steve Scherf
- *   Email: steve@moonsoft.com
- *   Moondog Software Productions - makers of fine public domain software.
+ *   Copyright (C) 1996-1997  Steve Scherf (steve@moonsoft.com)
+ *   Portions Copyright (C) 2001-2006  by various authors
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,28 +16,25 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
 
 #ifndef LINT
-static char *_sites_c_ident_ = "@(#)$Id: xmit.c,v 1.8 2001/09/08 18:16:54 gonter Exp $";
+static char *const _sites_c_ident_ = "@(#)$Id: xmit.c,v 1.18.2.3 2006/04/18 22:10:24 joerg78 Exp $";
 #endif
 
 #include <stdlib.h>
 #include <sys/wait.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
 #include <fcntl.h>
-#include <stdio.h>
 #include <errno.h>
 #include <dirent.h>
 #include <time.h>
 #include <unistd.h>
 #include <netinet/in.h>
 #include "access.h"
-#include "list.h"
 #include "cddbd.h"
 
 
@@ -287,6 +283,14 @@ getsiteent(int type)
 					case 'm':
 						site.st_flags |= ST_FLAG_OMIME;
 						break;
+						
+					case 'i':
+						site.st_flags |= ST_FLAG_ONLYISO;
+						break;
+					
+					case 'f':
+					site.st_flags |= ST_FLAG_OFIELDS;
+						break;
 
 					default:
 						break;
@@ -691,10 +695,15 @@ cddbd_do_transmit(csite_t *sp)
 	int fail;
 	int tcnt;
 	int files;
+	int use_proto = 1;
+	int db_enc=-1;
 	int flags= MF_NONE;
+	int dbflags;
 	db_t *db;
 	time_t xtime;
 	FILE *fp;
+	char *tmail;
+	FILE *tfp= (FILE *) 0;
 	lhead_t *li;
 	ct_key_t *key;
 	struct stat sbuf;
@@ -704,7 +713,8 @@ cddbd_do_transmit(csite_t *sp)
 	char subj[CDDBBUFSIZ];
 	char errstr[CDDBBUFSIZ];
 	char xmit_email[CDDBBUFSIZ];
-
+	
+	
 	if(sp->st_flags & ST_FLAG_NOXMIT) {
 		cddbd_log(LOG_ERR | LOG_XMIT,
 		    "Remote host %s is not configured for transmits.",
@@ -748,6 +758,18 @@ cddbd_do_transmit(csite_t *sp)
 
 			return;
 		}
+		if (!(use_proto=cddbp_setproto())) {
+			cddbd_log(LOG_ERR | LOG_XMIT,
+			    "Failed to determine and set protocol level on %s for file transmission.",
+			    sp->st_name);
+
+			if(!cddbd_close_history()) {
+				cddbd_log(LOG_ERR | LOG_XMIT,
+				    "Failed to update the history for: %s.",
+				    sp->st_name);
+			}
+			return;
+		}
 
 		break;
 
@@ -760,16 +782,12 @@ cddbd_do_transmit(csite_t *sp)
 
 				return;
 			}
-
+		}
+		/* This is for supporting older sites. */
+		if(sp->st_flags & ST_FLAG_OMIME)
+			flags = MF_ENC;
+		else
 			flags = MF_MULTI;
-		}
-		else {
-			/* This is for supporting older sites. */
-			if(sp->st_flags & ST_FLAG_OMIME)
-				flags = MF_ENC;
-			else
-				flags = MF_MULTI;
-		}
 
 		if(!smtp_open()) {
 			cddbd_log(LOG_ERR | LOG_XMIT,
@@ -850,13 +868,20 @@ cddbd_do_transmit(csite_t *sp)
 			quit(QUIT_ERR);
 		}
 
+#ifdef DB_WINDOWS_FORMAT
+		dbflags = file_df_flags | DF_WIN;
+		if((fp = db_prepare_unix_read(file)) != NULL) {
+#else
+		dbflags = file_df_flags;
 		if((fp = fopen(file, "r")) == NULL) {
+#endif
 			cddbd_log(LOG_ERR | LOG_XMIT,
-			    "Failed to open DB file %s for transmission.",file);
+			    "Transmission failed for %s %08x: No such CD entry in database.", dir, discid);
 			continue;
 		}
 
-		db = db_read(fp, errstr, 0);
+		db = db_read(fp, errstr, dbflags);
+		fclose(fp);
 
 		if(db == 0) {
 			switch(db_errno) {
@@ -876,13 +901,12 @@ cddbd_do_transmit(csite_t *sp)
 				break;
 			}
 
-			fclose(fp);
 			continue;
 		}
 
 		switch(sp->st_proto) {
 		case ST_PROTO_CDDBP:
-			switch(cddbp_transmit(db, dir, discid)) {
+			switch(cddbp_transmit(db, dir, discid, use_proto)) {
 			case -1:
 				/* DB file was corrupt. */
 				break;
@@ -902,12 +926,70 @@ cddbd_do_transmit(csite_t *sp)
 			break;
 
 		case ST_PROTO_SMTP:
-			rewind(fp);
 
 			cddbd_snprintf(subj, sizeof(subj), "cddb %s %08x",
 			    dir, discid);
-
-			if(!smtp_transmit(fp, subj, xmit_email, xmit_email,
+			
+			/**************************************************/
+			/* check whether the remote server supports utf-8 */
+			/* (this is set via a flag in the sites-file),    */
+			/* convert charset accordingly and set db_enc     */
+			/* variable (so the email can get the correct     */
+			/* charset-header later                           */
+			/**************************************************/
+			if(sp->st_flags & ST_FLAG_ONLYISO) {
+				convert_db_charset_proto(db, UNICODE_LEVEL-1);
+				if (db->db_flags & DB_ENC_ASCII)
+					db_enc=CC_US_ASCII;
+				else if (db->db_flags & DB_ENC_LATIN1)
+					db_enc=CC_ISO_8859;
+				else db_enc=-1;
+			}
+			else {
+				convert_db_charset_proto(db, UNICODE_LEVEL);
+				db_enc=CC_UTF_8;
+			}
+			
+			/**********************************************/
+			/* set use_proto depending on whether or not  */
+			/* we may send the DYEAR and DGENRE lines     */
+			/* (this is set via a flag in the sites file) */
+			/**********************************************/
+			if(sp->st_flags & ST_FLAG_OFIELDS)
+				use_proto=4;
+			else
+				use_proto=MAX_PROTO_LEVEL;
+			
+			tmail = cddbd_mktemp();
+			
+			/*********************************************/
+			/* Write the database entry to a tmp-file    */
+			/* so we can send it via email               */
+			/* (smtp_transmits needs the entry in a file */
+			/*********************************************/
+			if((tfp = fopen(tmail, "w+")) == NULL) {
+				cddbd_log(LOG_ERR | LOG_XMIT,
+		    	"Can't open mail tmp file %s (%d)", tmail, errno);
+		    	fail++;
+		    	break;
+			}
+			
+#ifdef DB_WINDOWS_FORMAT
+			if(db_write(tfp, db, use_proto, 0) == 0) {
+#else
+			if(db_write(tfp, db, use_proto) == 0) {
+#endif
+				cddbd_log(LOG_ERR | LOG_XMIT,
+				    "Can't write mail tmp file: %s", tmail);
+				fclose(tfp);
+				unlink(tmail);
+				break;
+			}
+			/* Put the pointer back to the start of the file for reading. */
+			rewind(tfp);
+			
+			/* try to send the transmission email */
+			if(!smtp_transmit(tfp, db_enc, subj, xmit_email, xmit_email,
 			    admin_email, 0, flags, (int) sbuf.st_size, key))
 				fail++;
 			else {
@@ -918,7 +1000,9 @@ cddbd_do_transmit(csite_t *sp)
 				if(email_time > 0)
 					cddbd_delay(email_time);
 			}
-
+			if (tfp != (FILE *) 0) fclose(tfp);
+			cddbd_freetemp(tmail);
+			
 			break;
 
 		default:
@@ -935,7 +1019,6 @@ cddbd_do_transmit(csite_t *sp)
 			quit(QUIT_ERR);
 		}
 
-		fclose(fp);
 		db_free(db);
 
 		if(fail) {
@@ -1187,8 +1270,8 @@ cddbd_put_file(char *site, char *dir, char *file)
 	int ret;
 	FILE *fp;
 	DIR *dirp;
-	struct stat sbuf;
 	struct dirent *dp;
+	struct stat sbuf;
 	char pfile[CDDBBUFSIZ];
 	char *bfile;
 
@@ -1285,9 +1368,9 @@ cddbd_do_catchup(char *site, char *when)
 	int i;
 	time_t t;
 	DIR *dirp;
+	struct dirent *dp;
 	csite_t *sp;
 	struct stat sbuf;
-	struct dirent *dp;
 	char buf[CDDBBUFSIZ];
 	char cdir[CDDBBUFSIZ];
 	char file[CDDBBUFSIZ];
